# -*- encoding: utf-8 -*-
"""Stats representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_date_attrib


class TaskReliability(_BaseEvergreenObject):
    """Representation of an Evergreen task reliability object."""

    test_file = evg_attrib('test_file')
    task_name = evg_attrib('task_name')
    variant = evg_attrib('variant')
    distro = evg_attrib('distro')
    evg_date_attrib('date')
    num_success = evg_attrib('num_success')
    num_failed = evg_attrib('num_failed')
    num_total = evg_attrib('num_total')
    num_timeout = evg_attrib('num_timeout')
    num_test_failed = evg_attrib('num_test_failed')
    num_system_failed = evg_attrib('num_system_failed')
    num_setup_failed = evg_attrib('num_setup_failed')
    avg_duration_pass = evg_attrib('avg_duration_pass')
    success_rate = evg_attrib('success_rate')

    def __init__(self, json, api):
        """
        Create an instance of a test stats object.

        :param json: json version of object.
        """
        super(TaskReliability, self).__init__(json, api)



from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_datetime_attrib


class GithubPatchData(_BaseEvergreenObject):
    """Representation of github patch data in a patch object."""

    pr_number = evg_attrib('pr_number')
    base_owner = evg_attrib('base_owner')
    base_repo = evg_attrib('base_repo')
    head_owner = evg_attrib('head_owner')
    head_repo = evg_attrib('head_repo')
    head_hash = evg_attrib('head_hash')
    author = evg_attrib('author')

    def __init__(self, json, api):
        """
        Create an instance of github patch data.

        :param json: json representing github patch data.
        :param api: instance of evergreen api object.
        """
        super(GithubPatchData, self).__init__(json, api)


class VariantsTasks(_BaseEvergreenObject):
    """Representation of a variants tasks object."""

    name = evg_attrib('name')

    def __init__(self, json, api):
        """
        Create an instance of a variants tasks object.

        :param json: json representing variants tasks object.
        :param api: intance of the evergreen api object.
        """
        super(VariantsTasks, self).__init__(json, api)
        self._task_set = 0

    @property
    def tasks(self):
        """
        Retrieve the set of all tasks for this variant.

        :return: Set of all tasks for this variant.
        """
        if not self._task_set:
            self._task_set = set(self.json['tasks'])

        return self._task_set


class Patch(_BaseEvergreenObject):
    """Representation of an Evergreen patch."""

    patch_id = evg_attrib('patch_id')
    description = evg_attrib('description')
    project_id = evg_attrib('project_id')
    branch = evg_attrib('branch')
    git_hash = evg_attrib('git_hash')
    patch_number = evg_attrib('patch_number')
    author = evg_attrib('author')
    version = evg_attrib('version')
    status = evg_attrib('status')
    create_time = evg_datetime_attrib('create_time')
    start_time = evg_datetime_attrib('start_time')
    finish_time = evg_datetime_attrib('finish_time')
    builds = evg_attrib('builds')
    tasks = evg_attrib('tasks')
    activated = evg_attrib('activated')
    alias = evg_attrib('alias')

    def __init__(self, json, api):
        """
        Create an instance of an evergreen patch.

        :param json: json representing patch.
        """
        super(Patch, self).__init__(json, api)
        self._variants_tasks = None
        self._variant_task_dict = None

    @property
    def github_patch_data(self):
        """
        Retrieve the github patch data for this patch.

        :return: Github Patch Data for this patch.
        """
        return GithubPatchData(self.json['github_patch_data'], self._api)

    @property
    def variants_tasks(self):
        """
        Retrieve the variants tasks for this patch.

        :return: variants tasks for this patch.
        """
        if not self._variants_tasks:
            self._variants_tasks = [
                VariantsTasks(vt, self._api) for vt in self.json['variants_tasks']
            ]
        return self._variants_tasks

    def task_list_for_variant(self, variant):
        """
        Retrieve the list of tasks for the given variant.

        :param variant: name of variant to search for.
        :return: list of tasks belonging to the specified variant.
        """
        if not self._variant_task_dict:
            self._variant_task_dict = {vt.name: vt.tasks for vt in self.variants_tasks}
        return self._variant_task_dict[variant]

    def get_version(self):
        """
        Get version for this patch.

        :return: Version object.
        """
        return self._api.version_by_id(self.version)

    def __str__(self):
        return '{}: {}'.format(self.patch_id, self.description)





class EvergreenException(Exception):
    """An exception coming from the evergreen client."""

    def __init__(self, msg=None):
        """
        Create a new exception instance.
        :param msg: Message describing exception.
        """
        if not msg:
            msg = 'Exception in evergreen client'

        super(EvergreenException, self).__init__(msg)


class MetricsException(EvergreenException):
    """An exception with metrics collection."""

    def __init__(self, msg=None):
        """
        Create a new exception instance.

        :param msg: Message describing exception.
        """
        if not msg:
            msg = 'Exception in metrics collection'

        super(MetricsException, self).__init__(msg)


class ActiveTaskMetricsException(MetricsException):
    """An exception when a task is in progress during metrics collection."""

    def __init__(self, task, msg=None):
        """
        Create a new exception instance.

        :param task: Task in progress.
        :param msg: Message describing exception.
        """
        if not msg:
            msg = 'Exception in metrics collection'

        super(MetricsException, self).__init__(msg)

        self.task = task



# -*- encoding: utf-8 -*-
"""Task representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_datetime_attrib

EVG_SUCCESS_STATUS = 'success'
EVG_SYSTEM_FAILURE_STATUS = 'system'
EVG_UNDISPATCHED_STATUS = 'undispatched'

_EVG_DATE_FIELDS_IN_TASK = frozenset([
    'create_time',
    'dispatch_time',
    'finish_time',
    'ingest_time',
    'scheduled_time',
    'start_time',
])


class Artifact(_BaseEvergreenObject):
    """Representation of a task artifact from evergreen."""

    name = evg_attrib('name')
    url = evg_attrib('url')
    visibility = evg_attrib('visibility')
    ignore_for_fetch = evg_attrib('ignore_for_fetch')

    def __init__(self, json, api):
        """Create an instance of an evergreen task artifact."""
        super(Artifact, self).__init__(json, api)


class StatusDetails(_BaseEvergreenObject):
    """Representation of a task status details from evergreen."""

    status = evg_attrib('status')
    type = evg_attrib('type')
    desc = evg_attrib('desc')
    timed_out = evg_attrib('timed_out')

    def __init__(self, json, api):
        """Create an instance of an evergreen task status details."""
        super(StatusDetails, self).__init__(json, api)


class Task(_BaseEvergreenObject):
    """Representation of an Evergreen task."""

    activated = evg_attrib('activated')
    activated_by = evg_attrib('activated_by')
    build_id = evg_attrib('build_id')
    build_variant = evg_attrib('build_variant')
    create_time = evg_datetime_attrib('create_time')
    depends_on = evg_attrib('depends_on')
    dispatch_time = evg_datetime_attrib('dispatch_time')
    display_name = evg_attrib('display_name')
    display_only = evg_attrib('display_only')
    distro_id = evg_attrib('distro_id')
    est_wait_to_start_ms = evg_attrib('est_wait_to_start_ms')
    estimated_cost = evg_attrib('estimated_cost')
    execution = evg_attrib('execution')
    expected_duration_ms = evg_attrib('expected_duration_ms')
    finish_time = evg_datetime_attrib('finish_time')
    generate_task = evg_attrib('generate_task')
    generated_by = evg_attrib('generated_by')
    host_id = evg_attrib('host_id')
    ingest_time = evg_datetime_attrib('ingest_time')
    mainline = evg_attrib('mainline')
    order = evg_attrib('order')
    project_id = evg_attrib('project_id')
    priority = evg_attrib('priority')
    restarts = evg_attrib('restarts')
    revision = evg_attrib('revision')
    scheduled_time = evg_datetime_attrib('scheduled_time')
    start_time = evg_datetime_attrib('start_time')
    status = evg_attrib('status')
    task_group = evg_attrib('task_group')
    task_group_max_hosts = evg_attrib('task_group_max_hosts')
    task_id = evg_attrib('task_id')
    time_taken_ms = evg_attrib('time_taken_ms')
    version_id = evg_attrib('version_id')

    def __init__(self, json, api):
        """
        Create an instance of an evergreen task.
        """
        super(Task, self).__init__(json, api)
        self._logs_map = None

    @property
    def artifacts(self):
        """
        Retrieve the artifacts for the given task.
        :return: List of artifacts.
        """
        return [Artifact(artifact, self._api) for artifact in self.json['artifacts']]

    @property
    def log_map(self):
        """
        Retrieve a dict of all the logs.
        :return: Dictionary of the logs.
        """
        if not self._logs_map:
            self._logs_map = {key: value for key, value in self.json['logs'].items()}
        return self._logs_map

    def retrieve_log(self, log_name, raw=False):
        """
        Retrieve the contents of the specified log.

        :param log_name: Name of log to retrieve.
        :param raw: Retrieve raw version of log.
        :return: Contents of the specified log.
        """
        return self._api.retrieve_task_log(self.log_map[log_name], raw)

    @property
    def status_details(self):
        """
        Retrieve the status details for the given task.
        :return: Status details.
        """
        return StatusDetails(self.json['status_details'], self._api)

    def get_execution(self, execution):
        """
        Get the task info for the specified execution.

        :param execution: Index of execution.
        :return: Task info for specified execution.
        """
        if self.execution == execution:
            return self

        if 'previous_executions' in self.json:
            for task in self.json['previous_executions']:
                if task['execution'] == execution:
                    return Task(task, self._api)

        return None

    def wait_time(self):
        """
        Get the time taken until the task started running.

        :return: Time taken until task started running.
        """
        if self.start_time and self.ingest_time:
            return self.start_time - self.ingest_time
        return None

    def wait_time_once_unblocked(self):
        """
        Get the time taken until the task started running
        once it is unblocked by task dependencies.

        :return: Time taken until task started running.
        """
        if self.start_time and self.scheduled_time:
            return self.start_time - self.scheduled_time
        return None

    def is_success(self):
        """
        Whether task was successful.

        :return: True if task was successful.
        """
        return self.status == EVG_SUCCESS_STATUS

    def is_undispatched(self):
        """
        Whether the task was undispatched.

        :return: True is task was undispatched.
        """
        return self.status == EVG_UNDISPATCHED_STATUS

    def is_system_failure(self):
        """
        Whether task resulted in a system failure.

        :return: True if task was a system failure.
        """
        if not self.is_success() and self.status_details and self.status_details.type:
            return self.status_details.type == EVG_SYSTEM_FAILURE_STATUS
        return False

    def is_timeout(self):
        """
        Whether task results in a timeout.

        :return: True if task was a timeout.
        """
        if not self.is_success() and self.status_details and self.status_details.timed_out:
            return self.status_details.timed_out
        return False

    def is_active(self):
        """
        Determine if the given task is active.

        :return: True if task is active.
        """
        return self.scheduled_time and not self.finish_time



# -*- encoding: utf-8 -*-
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_datetime_attrib
from evergreen.metrics.buildmetrics import BuildMetrics


EVG_BUILD_STATUS_FAILED = 'failed'
EVG_BUILD_STATUS_SUCCESS = 'success'
EVG_BUILD_STATUS_CREATED = 'created'

COMPLETED_STATES = {
    EVG_BUILD_STATUS_FAILED,
    EVG_BUILD_STATUS_SUCCESS,
}


class StatusCounts(_BaseEvergreenObject):
    """Representation of Evergreen StatusCounts."""

    succeeded = evg_attrib('succeeded')
    failed = evg_attrib('failed')
    started = evg_attrib('started')
    undispatched = evg_attrib('undispatched')
    inactivate = evg_attrib('inactive')
    dispatched = evg_attrib('dispatched')
    timed_out = evg_attrib('timed_out')

    def __init__(self, json, api):
        super(StatusCounts, self).__init__(json, api)


class Build(_BaseEvergreenObject):
    """Representation of an Evergreen build."""

    id = evg_attrib('_id')
    project_id = evg_attrib('project_id')
    create_time = evg_datetime_attrib('create_time')
    start_time = evg_datetime_attrib('start_time')
    finish_time = evg_datetime_attrib('finish_time')
    version = evg_attrib('version')
    branch = evg_attrib('branch')
    git_hash = evg_attrib('git_hash')
    build_variant = evg_attrib('build_variant')
    status = evg_attrib('status')
    activated = evg_attrib('activated')
    activated_by = evg_attrib('activated_by')
    activated_time = evg_datetime_attrib('activated_time')
    order = evg_attrib('order')
    tasks = evg_attrib('tasks')
    time_taken_ms = evg_attrib('time_taken_ms')
    display_name = evg_attrib('display_name')
    predicted_makespan_ms = evg_attrib('predicted_makespan_ms')
    actual_makespan_ms = evg_attrib('actual_makespan_ms')
    origin = evg_attrib('origin')

    def __init__(self, json, api):
        """
        Create an instance of an evergreen task.
        """
        super(Build, self).__init__(json, api)

    @property
    def status_counts(self):
        return StatusCounts(self.json['status_counts'], self._api)

    def get_tasks(self, fetch_all_executions=False):
        """
        Get all tasks for this build.

        :param fetch_all_executions:  fetch all executions for tasks.
        :return: List of all tasks.
        """
        return self._api.tasks_by_build(self.id, fetch_all_executions)

    def is_completed(self):
        """
        Determine if this build has completed running tasks.

        :return: True if build has completed running tasks.
        """
        return self.status in COMPLETED_STATES

    def get_metrics(self, task_filter_fn=None):
        """
        Get metrics for the build.

        Metrics are only available on build that have finished running..

        :param task_filter_fn: function to filter tasks included for metrics, should accept a task
                               argument.
        :return: Metrics for the build.
        """
        if self.status != EVG_BUILD_STATUS_CREATED:
            return BuildMetrics(self).calculate(task_filter_fn)
        return None



# -*- encoding: utf-8 -*-
"""Project history representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib


class ProjectHistoryTask(_BaseEvergreenObject):
    """
    Representation of a project history task in evergreen.
    """
    task_id = evg_attrib('task_id')
    status = evg_attrib('status')
    time_taken = evg_attrib('time_taken')

    def __init__(self, name, json, api):
        """Create an instance of a project history task object."""
        super(ProjectHistoryTask, self).__init__(json, api)
        self.task_name = name


class ProjectHistoryBuild(_BaseEvergreenObject):
    """
    Representation of a project history build in evergreen.
    """
    build_id = evg_attrib('build_id')
    display_name = evg_attrib('name')

    def __init__(self, name, json, api):
        """Create an instance of a project history build object."""
        super(ProjectHistoryBuild, self).__init__(json, api)
        self.build_name = name

    @property
    def tasks(self):
        return [ProjectHistoryTask(key, value, self._api) for key, value in
                self.json['tasks'].items()]


class ProjectHistoryVersion(_BaseEvergreenObject):
    """
    Representation of a project history version in evergreen.
    """
    version_id = evg_attrib('version_id')
    author = evg_attrib('author')
    revision = evg_attrib('revision')
    message = evg_attrib('message')

    def __init__(self, json, api):
        """Create an instance of a project history version object."""
        super(ProjectHistoryVersion, self).__init__(json, api)

    @property
    def builds(self):
        return [ProjectHistoryBuild(key, value, self._api) for key, value in
                self.json['builds'].items()]


class ProjectHistory(_BaseEvergreenObject):
    """
    Representation of a project's history evergreen.
    """
    project = evg_attrib('project')

    def __init__(self, json, api):
        """Create an instance of a project's history object."""
        super(ProjectHistory, self).__init__(json, api)

    @property
    def versions(self):
        """
        Get the versions data.

        :return: versions data.
        """
        return [ProjectHistoryVersion(version, self._api) for version in self.json['versions']]



# -*- encoding: utf-8 -*-
"""Performance results representation of evergreen."""
from __future__ import absolute_import

import copy

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_short_datetime_attrib, \
    evg_datetime_attrib
from evergreen.util import parse_evergreen_datetime


class PerformanceTestResult(_BaseEvergreenObject):
    """Representation of a test result from Evergreen."""
    thread_level = evg_attrib('thread_level')
    operations_per_second = evg_attrib('ops_per_sec_values')
    mean_operations_per_second = evg_attrib('ops_per_sec')

    def __init__(self, json, api):
        """Create an instance of a test result."""
        super(PerformanceTestResult, self).__init__(json, api)


class PerformanceTestRun(_BaseEvergreenObject):
    """Representation of a test run from Evergreen."""
    workload = evg_attrib('workload')
    test_name = evg_attrib('name')

    def __init__(self, test_result, api):
        """Create an instance of a test run."""
        super(PerformanceTestRun, self).__init__(test_result, api)

        # Microbenchmarks stores the 'start' and 'end' time of the test in the inner 'results' field
        # while sys-perf stores it in the outer 'results' field.
        self.start = parse_evergreen_datetime(
            test_result.get('start', test_result['results']['start']))
        self.end = parse_evergreen_datetime(
            test_result.get('end', test_result['results']['end']))
        # Microbenchmarks does not produce a 'workload' field. We need to fill in the 'workload'
        # field for microbenchmark points in order to query on 'workload'.
        self.maximum_thread_level, self.maximum_operations_per_second = _get_max_ops_per_sec(
            test_result)

    @property
    def test_results(self):
        if self.maximum_operations_per_second is None:
            return None
        return _get_performance_results(self.json, self._api)


class PerformanceTestBatch(_BaseEvergreenObject):
    """Representation of a batch of tests from Evergreen."""
    start = evg_datetime_attrib('start')
    end = evg_datetime_attrib('end')
    storage_engine = evg_attrib('storageEngine')
    errors = evg_attrib('errors')

    def __init__(self, json, api, parent):
        """Create an instance of a batch of tests"""
        super(PerformanceTestBatch, self).__init__(json, api)
        self.parent = parent

    @property
    def test_runs(self):
        return [PerformanceTestRun(item, self._api) for item in self.json['results']]

    def test_runs_matching(self, tests):
        return [item for item in self.test_runs if
                _is_run_matching(item, tests)]


class PerformanceData(_BaseEvergreenObject):
    """Representation of performance data from Evergreen."""
    name = evg_attrib('name')
    project_id = evg_attrib('project_id')
    task_name = evg_attrib('task_name')
    task_id = evg_attrib('task_id')
    variant = evg_attrib('variant')
    version_id = evg_attrib('version_id')
    revision = evg_attrib('revision')
    order = evg_attrib('order')
    tag = evg_attrib('tag')
    create_time = evg_short_datetime_attrib('create_time')

    def __init__(self, json, api):
        """Create an instance of performance data"""
        super(PerformanceData, self).__init__(json, api)

    @property
    def test_batch(self):
        return PerformanceTestBatch(self.json['data'], self._api, self)


def _get_performance_results(test_result, api):
    """
    Extract and sort the thread level and respective results from the raw data file from Evergreen.
    See below for an example of the resulting format:

        [
            {
                'thread_level': '1',
                'ops_per_sec': 500,
                'ops_per_sec': [
                    500
                ]
            },
            {
                'thread_level: '2',
                'ops_per_sec': 700,
                'ops_per_sec': [
                    700
                ]
            }
        ]

    :param dict test_result: All the test results from the raw data file from Evergreen.
    :return: A list of dictionaries with test results organized by thread level.
    """
    thread_levels = []
    for thread_level, result in test_result['results'].items():
        if isinstance(result, dict):
            this_result = copy.deepcopy(result)
            this_result.pop('error_values', None)
            this_result.update({'thread_level': thread_level})
            test_result = PerformanceTestResult(this_result, api)
            thread_levels.append(test_result)
    return sorted(thread_levels, key=lambda k: k.thread_level)


def _get_max_ops_per_sec(test_result):
    """
    For a given set of test results, find and return the maximum operations per second metric and
    its respective thread level.

    :param dict test_result: All the test results from the raw data file from Evergreen.
    :return: The maximum operations per second found and its respective thread level.
    :rtype: tuple(int, int).
    """
    max_ops_per_sec = None
    max_thread_level = None
    results = test_result['results']
    for key, thread_level in results.items():
        if not key.isdigit():
            continue
        if max_ops_per_sec is None or max_ops_per_sec < thread_level['ops_per_sec']:
            max_ops_per_sec = thread_level['ops_per_sec']
            max_thread_level = int(key)
    return max_thread_level, max_ops_per_sec


def _is_run_matching(test_run, tests):
    """
    Determine if the given test_run matches a set of tests.

    :param test_run: test_run to check.
    :param tests: List of tests to upload.
    :return: True if the test_run contains relevant data.
    """
    if tests is not None and test_run.test_name not in tests:
        return False

    if test_run.start is None:
        return False

    if all(result.operations_per_second is None for result in test_run.test_results):
        return False

    return True



# -*- encoding: utf-8 -*-
"""API for interacting with evergreen."""
from __future__ import absolute_import

import time
from datetime import datetime, timedelta
from enum import Enum

from evergreen.performance_results import PerformanceData
from evergreen.project_history import ProjectHistory

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError  # JSONDecodeError doesn't exist in python 2, ValueError is used.

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse  # type: ignore

try:
    from functools import lru_cache
except ImportError:
    from backports.functools_lru_cache import lru_cache

import requests
import structlog
from structlog.stdlib import LoggerFactory
from tenacity import retry, stop_after_attempt, wait_exponential, retry_if_exception_type

from evergreen.build import Build
from evergreen.commitqueue import CommitQueue
from evergreen.config import read_evergreen_config, DEFAULT_API_SERVER, get_auth_from_config, \
    DEFAULT_NETWORK_TIMEOUT_SEC
from evergreen.distro import Distro
from evergreen.host import Host
from evergreen.manifest import Manifest
from evergreen.patch import Patch
from evergreen.project import Project
from evergreen.task import Task
from evergreen.tst import Tst
from evergreen.stats import TestStats, TaskStats
from evergreen.task_reliability import TaskReliability
from evergreen.util import evergreen_input_to_output, format_evergreen_datetime
from evergreen.version import Version

structlog.configure(logger_factory=LoggerFactory())
LOGGER = structlog.getLogger(__name__)

CACHE_SIZE = 5000
DEFAULT_LIMIT = 100
MAX_RETRIES = 3
START_WAIT_TIME_SEC = 2
MAX_WAIT_TIME_SEC = 5

Requester = Enum('Requester', 'PATCH_REQUEST GITTER_REQUEST GITHUB_PULL_REQUEST MERGE_TEST AD_HOC')


class _BaseEvergreenApi(object):
    """Base methods for building API objects."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """
        Create a _BaseEvergreenApi object.

        :param api_server: URI of Evergreen API server.
        :param auth: EvgAuth object with auth information.
        """
        self._timeout = timeout
        self._api_server = api_server
        self.session = requests.Session()
        adapter = requests.adapters.HTTPAdapter()
        self.session.mount('{url.scheme}://'.format(url=urlparse(api_server)), adapter)
        if auth:
            self.session.headers.update({
                'Api-User': auth.username,
                'Api-Key': auth.api_key,
            })

    def _create_url(self, endpoint):
        """
        Format the a call to a v2 REST API endpoint.

        :param endpoint: endpoint to call.
        :return: Full url to get endpoint.
        """
        return '{api_server}/rest/v2{endpoint}'.format(
            api_server=self._api_server, endpoint=endpoint)

    def _create_v1_url(self, endpoint):
        """
        Format the a call to a v1 REST API endpoint.

        :param endpoint: endpoint to call.
        :return: Full url to get endpoint.
        """
        return '{api_server}/rest/v1{endpoint}'.format(
            api_server=self._api_server, endpoint=endpoint)

    def _create_plugin_url(self, endpoint):
        """
        Format the a call to a plugin endpoint.

        :param endpoint: endpoint to call.
        :return: Full url to get endpoint.
        """
        return '{api_server}/plugin/json{endpoint}'.format(
            api_server=self._api_server, endpoint=endpoint)

    @staticmethod
    def _log_api_call_time(response, start_time):
        """
        Log how long the api call took.

        :param response: Response from API.
        :param start_time: Time the response was started.
        """
        duration = round(time.time() - start_time, 2)
        if duration > 10:
            LOGGER.info('Request completed.', url=response.request.url, duration=duration)
        else:
            LOGGER.debug('Request completed.', url=response.request.url, duration=duration)

    def _call_api(self, url, params=None):
        """
        Make a call to the evergreen api.

        :param url: Url of call to make.
        :param params: parameters to pass to api.
        :return: response from api server.
        """
        start_time = time.time()
        response = self.session.get(url=url, params=params, timeout=self._timeout)
        self._log_api_call_time(response, start_time)

        self._raise_for_status(response)
        return response

    @staticmethod
    def _raise_for_status(response):
        """
        Raise an exception with the evergreen message if it exists.

        :param response: response from evergreen api.
        """
        try:
            json_data = response.json()
            if response.status_code >= 400 and 'error' in json_data:
                raise requests.exceptions.HTTPError(json_data['error'], response=response)
        except JSONDecodeError:
            pass

        response.raise_for_status()

    def _paginate(self, url, params=None):
        """
        Paginate until all results are returned and return a list of all JSON results.

        :param url: url to make request to.
        :param params: parameters to pass to request.
        :return: json list of all results.
        """
        response = self._call_api(url, params)
        json_data = response.json()
        while "next" in response.links:
            if params and 'limit' in params and len(json_data) >= params['limit']:
                break
            response = self._call_api(response.links['next']['url'])
            if response.json():
                json_data.extend(response.json())

        return json_data

    def _lazy_paginate(self, url, params=None):
        """
        Lazy paginate, the results are returned lazily.

        :param url: URL to query.
        :param params: Params to pass to url.
        :return: A generator to get results from.
        """
        if not params:
            params = {
                'limit': DEFAULT_LIMIT,
            }

        next_url = url
        while True:
            response = self._call_api(next_url, params)
            json_response = response.json()
            if not json_response:
                break
            for result in json_response:
                yield result
            if 'next' not in response.links:
                break

            next_url = response.links['next']['url']

    def _lazy_paginate_by_date(self, url, params=None):
        """
        Paginate based on date, the results are returned lazily.

        :param url: URL to query.
        :param params: Params to pass to url.
        :return: A generator to get results from.
        """
        if not params:
            params = {
                'limit': DEFAULT_LIMIT,
            }

        while True:
            data = self._call_api(url, params).json()
            if not data:
                break
            for result in data:
                yield result
            params['start_at'] = evergreen_input_to_output(data[-1]['create_time'])


class _DistrosApi(_BaseEvergreenApi):
    """API for distros endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_DistrosApi, self).__init__(api_server, auth, timeout)

    def all_distros(self):
        """
        Get all distros in evergreen.

        :return: List of all distros in evergreen.
        """
        url = self._create_url('/distros')
        distro_list = self._paginate(url)
        return [Distro(distro, self) for distro in distro_list]


class _HostApi(_BaseEvergreenApi):
    """API for hosts endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_HostApi, self).__init__(api_server, auth, timeout)

    def all_hosts(self, status=None):
        """
        Get all hosts in evergreen.

        :param status: Only return hosts with specified status.
        :return: List of all hosts in evergreen.
        """
        params = {}
        if status:
            params['status'] = status

        url = self._create_url('/hosts')
        host_list = self._paginate(url, params)
        return [Host(host, self) for host in host_list]


class _ProjectApi(_BaseEvergreenApi):
    """API for project endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_ProjectApi, self).__init__(api_server, auth, timeout)

    def all_projects(self):
        """
        Get all projects in evergreen.

        :return: List of all projects in evergreen.
        """
        url = self._create_url('/projects')
        project_list = self._paginate(url)
        return [Project(project, self) for project in project_list]

    def project_by_id(self, project_id):
        """
        Get a project by project_id.

        :param project_id: Id of project to query.
        :return: Project specified.
        """
        url = self._create_url('/projects/{project_id}'.format(project_id=project_id))
        return Project(self._paginate(url), self)

    def recent_version_by_project(self, project_id, params=None):
        """
        Get recent versions created in specified project.

        :param project_id: Id of project to query.
        :param params: parameters to pass to endpoint.
        :return: List of recent versions.
        """
        url = self._create_url(
            '/projects/{project_id}/recent_versions'.format(project_id=project_id))
        version_list = self._paginate(url, params)
        return [Version(version, self) for version in version_list]

    def versions_by_project(self, project_id, requester=Requester.GITTER_REQUEST):
        """
        Get the versions created in the specified project.

        :param project_id: Id of project to query.
        :param requester: Type of versions to query.
        :return: Generator of versions.
        """
        url = self._create_url('/projects/{project_id}/versions'.format(project_id=project_id))
        params = {
            'requester': requester.name.lower()
        }
        version_list = self._lazy_paginate(url, params)
        return (Version(version, self) for version in version_list)

    def patches_by_project(self, project_id, params=None):
        """
        Get a list of patches for the specified project.

        :param project_id: Id of project to query.
        :param params: parameters to pass to endpoint.
        :return: List of recent patches.
        """
        url = self._create_url('/projects/{project_id}/patches'.format(project_id=project_id))
        patches = self._lazy_paginate_by_date(url, params)
        return (Patch(patch, self) for patch in patches)

    def commit_queue_for_project(self, project_id):
        """
        Get the current commit queue for the specified project.

        :param project_id: Id of project to query.
        :return: Current commit queue for project.
        """
        url = self._create_url('/commit_queue/{project_id}'.format(project_id=project_id))
        return CommitQueue(self._paginate(url), self)

    def test_stats_by_project(self,
                              project_id,
                              after_date,
                              before_date,
                              group_num_days=None,
                              requesters=None,
                              tests=None,
                              tasks=None,
                              variants=None,
                              distros=None,
                              group_by=None,
                              sort=None):
        """
        Get a patch by patch id.

        :param project_id: Id of patch to query for.
        :param after_date: Collect stats after this date.
        :param before_date: Collect stats before this date.
        :param group_num_days: Aggregate statistics to this size.
        :param requesters: Filter by requestors (mainline, patch, trigger, or adhoc).
        :param tests: Only include specified tests.
        :param tasks: Only include specified tasks.
        :param variants: Only include specified variants.
        :param distros: Only include specified distros.
        :param group_by: How to group results (test_task_variant, test_task, or test)
        :param sort: How to sort results (earliest or latest).
        :return: Patch queried for.
        """
        params = {
            'after_date': after_date,
            'before_date': before_date,
        }
        if group_num_days:
            params['group_num_days'] = group_num_days
        if requesters:
            params['requesters'] = requesters
        if tests:
            params['tests'] = tests
        if tasks:
            params['tasks'] = tasks
        if variants:
            params['variants'] = variants
        if distros:
            params['distros'] = distros
        if group_by:
            params['group_by'] = group_by
        if sort:
            params['sort'] = sort
        url = self._create_url('/projects/{project_id}/test_stats'.format(project_id=project_id))
        test_stats_list = self._paginate(url, params)
        return [TestStats(test_stat, self) for test_stat in test_stats_list]

    def tasks_by_project(self, project_id, statuses=None):
        """
        Get all the tasks for a project.

        :param project_id: The project's id.
        :param statuses: the types of statuses to get tasks for.
        :return: The list of matching tasks.
        """
        url = self._create_url(
            "/projects/{project_id}/versions/tasks".format(project_id=project_id))
        params = {'status': statuses} if statuses else None
        return [Task(json, self) for json in self._paginate(url, params)]

    def project_history(self, project_id):
        """
        Get a project's history

        :param project_id: The project's id.
        :return: The project's history
        """
        url = self._create_v1_url("/projects/{project_id}/versions".format(project_id=project_id))
        return ProjectHistory(self._paginate(url), self)

    def task_stats_by_project(self,
                              project_id,
                              after_date,
                              before_date,
                              group_num_days=None,
                              requesters=None,
                              tasks=None,
                              variants=None,
                              distros=None,
                              group_by=None,
                              sort=None):
        """
        Get task stats by project id.

        :param project_id: Id of patch to query for.
        :param after_date: Collect stats after this date.
        :param before_date: Collect stats before this date.
        :param group_num_days: Aggregate statistics to this size.
        :param requesters: Filter by requestors (mainline, patch, trigger, or adhoc).
        :param tasks: Only include specified tasks.
        :param variants: Only include specified variants.
        :param distros: Only include specified distros.
        :param group_by: How to group results (test_task_variant, test_task, or test)
        :param sort: How to sort results (earliest or latest).
        :return: Patch queried for.
        """
        params = {
            'after_date': after_date,
            'before_date': before_date,
        }
        if group_num_days:
            params['group_num_days'] = group_num_days
        if requesters:
            params['requesters'] = requesters
        if tasks:
            params['tasks'] = tasks
        if variants:
            params['variants'] = variants
        if distros:
            params['distros'] = distros
        if group_by:
            params['group_by'] = group_by
        if sort:
            params['sort'] = sort
        url = self._create_url('/projects/{project_id}/task_stats'.format(project_id=project_id))
        task_stats_list = self._paginate(url, params)
        return [TaskStats(task_stat, self) for task_stat in task_stats_list]

    def task_reliability_by_project(self,
                                    project_id,
                                    after_date=None,
                                    before_date=None,
                                    group_num_days=None,
                                    requesters=None,
                                    tasks=None,
                                    variants=None,
                                    distros=None,
                                    group_by=None,
                                    sort=None):
        """
        Get task reliability scores.

        :param project_id: Id of patch to query for.
        :param after_date: Collect stats after this date.
        :param before_date: Collect stats before this date, defaults to nothing.
        :param group_num_days: Aggregate statistics to this size.
        :param requesters: Filter by requesters (mainline, patch, trigger, or adhoc).
        :param tasks: Only include specified tasks.
        :param variants: Only include specified variants.
        :param distros: Only include specified distros.
        :param group_by: How to group results (test_task_variant, test_task, or test)
        :param sort: How to sort results (earliest or latest).
        :return: Patch queried for.
        """
        if after_date is None:
            date = datetime.utcnow() - timedelta(days=180)
            after_date = format_evergreen_datetime(date)

        params = {'after_date': after_date}
        if before_date:
            params['before_date'] = before_date
        if group_num_days:
            params['group_num_days'] = group_num_days
        if requesters:
            params['requesters'] = requesters
        if tasks:
            params['tasks'] = tasks
        if variants:
            params['variants'] = variants
        if distros:
            params['distros'] = distros
        if group_by:
            params['group_by'] = group_by
        if sort:
            params['sort'] = sort
        url = self._create_url('/projects/{project_id}/task_reliability'.format(
            project_id=project_id))
        task_reliability_scores = self._paginate(url, params)
        return [TaskReliability(task_reliability, self)
                for task_reliability in task_reliability_scores]


class _BuildApi(_BaseEvergreenApi):
    """API for build endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_BuildApi, self).__init__(api_server, auth, timeout)

    def build_by_id(self, build_id):
        """
        Get a build by id.

        :param build_id: build id to query.
        :return: Build queried for.
        """
        url = self._create_url('/builds/{build_id}'.format(build_id=build_id))
        return Build(self._paginate(url), self)

    def tasks_by_build(self, build_id, fetch_all_executions=None):
        """
        Get all tasks for a given build.

        :param build_id: build_id to query.
        :param fetch_all_executions: Fetch all executions for a given task.
        :return: List of tasks for the specified build.
        """
        params = {}
        if fetch_all_executions:
            params['fetch_all_executions'] = 1

        url = self._create_url('/builds/{build_id}/tasks'.format(build_id=build_id))
        task_list = self._paginate(url, params)
        return [Task(task, self) for task in task_list]


class _VersionApi(_BaseEvergreenApi):
    """API for version endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_VersionApi, self).__init__(api_server, auth, timeout)

    def version_by_id(self, version_id):
        """
        Get version by version id.

        :param version_id: Id of version to query.
        :return: Version queried for.
        """
        url = self._create_url('/versions/{version_id}'.format(version_id=version_id))
        return Version(self._paginate(url), self)

    def builds_by_version(self, version_id, params=None):
        """
        Get all builds for a given Evergreen version_id.

        :param version_id: Version Id to query for.
        :param params: Dictionary of parameters to pass to query.
        :return: List of builds for the specified version.
        """
        url = self._create_url('/versions/{version_id}/builds'.format(version_id=version_id))
        build_list = self._paginate(url, params)
        return [Build(build, self) for build in build_list]


class _PatchApi(_BaseEvergreenApi):
    """API for patch endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_PatchApi, self).__init__(api_server, auth, timeout)

    def patch_by_id(self, patch_id, params=None):
        """
        Get a patch by patch id.

        :param patch_id: Id of patch to query for.
        :param params: Parameters to pass to endpoint.
        :return: Patch queried for.
        """
        url = self._create_url('/patches/{patch_id}'.format(patch_id=patch_id))
        return Patch(self._call_api(url, params).json(), self)


class _TaskApi(_BaseEvergreenApi):
    """API for task endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_TaskApi, self).__init__(api_server, auth, timeout)

    def task_by_id(self, task_id, fetch_all_executions=None):
        """
        Get a task by task_id.

        :param task_id: Id of task to query for.
        :param fetch_all_executions: Should all executions of the task be fetched.
        :return: Task queried for.
        """
        params = None
        if fetch_all_executions:
            params = {'fetch_all_executions': fetch_all_executions}
        url = self._create_url('/tasks/{task_id}'.format(task_id=task_id))
        return Task(self._call_api(url, params).json(), self)

    def tests_by_task(self, task_id, status=None, execution=None):
        """
        Get all tests for a given task.

        :param task_id: Id of task to query for.
        :param status: Limit results to given status.
        :param execution: Retrieve the specified task execution (defaults to 0).
        :return: List of tests for the specified task.
        """
        params = {}
        if status:
            params['status'] = status
        if execution:
            params['execution'] = execution
        url = self._create_url('/tasks/{task_id}/tests'.format(task_id=task_id))
        return [Tst(test, self) for test in self._paginate(url, params)]

    def performance_results_by_task(self, task_id):
        """
        Get the 'perf.json' performance results for a given task_id

        :param task_id: Id of task to query for.
        :return: Contents of 'perf.json'
        """
        url = self._create_plugin_url('/task/{task_id}/perf'.format(task_id=task_id))
        return PerformanceData(self._paginate(url), self)

    def performance_results_by_task_name(self, task_id, task_name):
        """
        Get the 'perf.json' performance results for a given task_id and task_name

        :param task_id: Id of task to query for.
        :param task_name: Name of task to query for.
        :return: Contents of 'perf.json'
        """
        url = '{api_server}/api/2/task/{task_id}/json/history/{task_name}/perf'.format(
            api_server=self._api_server, task_id=task_id, task_name=task_name)
        return [PerformanceData(result, self) for result in self._paginate(url)]


class _OldApi(_BaseEvergreenApi):
    """API for pre-v2 endpoints."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_OldApi, self).__init__(api_server, auth, timeout)

    def _create_old_url(self, endpoint):
        """
        Build a url for an pre-v2 endpoint.

        :param endpoint: endpoint to build url for.
        :return: An string pointing to the given endpoint.
        """
        return '{api_server}/{endpoint}'.format(api_server=self._api_server, endpoint=endpoint)

    def manifest(self, project_id, revision):
        """
        Get the manifest for the given revision.

        :param project_id: Project the revision belongs to.
        :param revision: Revision to get manifest of.
        :return: Manifest of the given revision of the given project.
        """
        url = self._create_old_url('plugin/manifest/get/{project_id}/{revision}'.format(
            project_id=project_id, revision=revision))
        return Manifest(self._call_api(url).json(), self)


class _LogApi(_BaseEvergreenApi):
    """API for accessing log files."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(_LogApi, self).__init__(api_server, auth, timeout)

    def retrieve_task_log(self, log_url, raw=False):
        """
        Get the request log file from a task.

        :param log_url: URL of log to retrieve.
        :param raw: Retrieve the raw version of the log
        :return: Contents of specified log file.
        """
        params = {}
        if raw:
            params['text'] = 'true'
        return self._call_api(log_url, params=params).text


class EvergreenApi(_ProjectApi, _BuildApi, _VersionApi, _PatchApi, _HostApi, _TaskApi, _OldApi,
                   _LogApi, _DistrosApi):
    """Access to the Evergreen API Server."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(EvergreenApi, self).__init__(api_server, auth, timeout=timeout)

    @classmethod
    def get_api(cls, auth=None, use_config_file=False, config_file=None,
                timeout=DEFAULT_NETWORK_TIMEOUT_SEC):
        """
        Get an evergreen api instance based on config file settings.

        :param auth: EvgAuth with authentication to use.
        :param use_config_file: attempt to read auth from default config file.
        :param config_file: config file with authentication information.
        :param timeout: Network timeout.
        :return: EvergreenApi instance.
        """
        kwargs = {'auth': auth, 'timeout': timeout}
        if not auth and use_config_file:
            config = read_evergreen_config()
            auth = get_auth_from_config(config)
            if auth:
                kwargs['auth'] = auth

            # If there is a value for api_server_host, then use it.
            if 'evergreen' in config and config['evergreen'].get('api_server_host', None):
                kwargs['api_server'] = config['evergreen']['api_server_host']

        return cls(**kwargs)


class CachedEvergreenApi(EvergreenApi):
    """
    Access to the Evergreen API server that caches certain calls.
    """

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(CachedEvergreenApi, self).__init__(api_server, auth, timeout)

    @lru_cache(maxsize=CACHE_SIZE)
    def build_by_id(self, build_id):
        """
        Get a build by id.

        :param build_id: build id to query.
        :return: Build queried for.
        """
        return super(CachedEvergreenApi, self).build_by_id(build_id)

    @lru_cache(maxsize=CACHE_SIZE)
    def version_by_id(self, version_id):
        """
        Get version by version id.

        :param version_id: Id of version to query.
        :return: Version queried for.
        """
        return super(CachedEvergreenApi, self).version_by_id(version_id)

    def clear_caches(self):
        """
        Clear the cache.
        """
        cached_functions = [
            self.build_by_id,
            self.version_by_id,
        ]
        for fn in cached_functions:
            fn.cache_clear()


class RetryingEvergreenApi(EvergreenApi):
    """An Evergreen Api that retries failed calls."""

    def __init__(self, api_server=DEFAULT_API_SERVER, auth=None, timeout=None):
        """Create an Evergreen Api object."""
        super(RetryingEvergreenApi, self).__init__(api_server, auth, timeout)

    @retry(retry=retry_if_exception_type(requests.exceptions.HTTPError),
           stop=stop_after_attempt(MAX_RETRIES),
           wait=wait_exponential(multiplier=1, min=START_WAIT_TIME_SEC, max=MAX_WAIT_TIME_SEC))
    def _call_api(self, url, params=None):
        """
        Call into the evergreen api.

        :param url: Url to call.
        :param params: Parameters to pass to api.
        :return: Result from calling API.
        """
        return super(RetryingEvergreenApi, self)._call_api(url, params)



# -*- encoding: utf-8 -*-
"""Version representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_datetime_attrib
from evergreen.metrics.versionmetrics import VersionMetrics


EVG_VERSION_STATUS_SUCCESS = 'success'
EVG_VERSION_STATUS_FAILED = 'failed'
EVG_VERSION_STATUS_CREATED = 'created'

COMPLETED_STATES = {
    EVG_VERSION_STATUS_FAILED,
    EVG_VERSION_STATUS_SUCCESS,
}


class BuildVariantStatus(_BaseEvergreenObject):
    """Representation of a Build Variants status."""

    build_variant = evg_attrib('build_variant')
    build_id = evg_attrib('build_id')

    def __init__(self, json, api):
        """Create an instance of a Build Variants status."""
        super(BuildVariantStatus, self).__init__(json, api)

    def get_build(self):
        """Get the build object for this build variants status."""
        return self._api.build_by_id(self.build_id)


class Version(_BaseEvergreenObject):
    """Representation of a Evergreen Version."""

    version_id = evg_attrib('version_id')
    create_time = evg_datetime_attrib('create_time')
    start_time = evg_datetime_attrib('start_time')
    finish_time = evg_datetime_attrib('finish_time')
    revision = evg_attrib('revision')
    order = evg_attrib('order')
    project = evg_attrib('project')
    author = evg_attrib('author')
    author_email = evg_attrib('author_email')
    message = evg_attrib('message')
    status = evg_attrib('status')
    repo = evg_attrib('repo')
    branch = evg_attrib('branch')
    errors = evg_attrib('errors')
    warnings = evg_attrib('warnings')
    ignored = evg_attrib('ignored')

    def __init__(self, json, api):
        """
        Create an instance of an evergreen version.

        :param json: json representing version
        """
        super(Version, self).__init__(json, api)

        if 'build_variants_status' in self.json and self.json['build_variants_status']:
            self.build_variants_map = {
                bvs['build_variant']: bvs['build_id']
                for bvs in self.json['build_variants_status']
            }

    @property
    def build_variants_status(self):
        if 'build_variants_status' not in self.json or not self.json['build_variants_status']:
            return []
        build_variants_status = self.json['build_variants_status']
        return [BuildVariantStatus(bvs, self._api) for bvs in build_variants_status]

    def build_by_variant(self, build_variant):
        """
        Get a build object for the specified variant.

        :param build_variant: Build variant to get build for.
        :return: Build object for variant.
        """
        return self._api.build_by_id(self.build_variants_map[build_variant])

    def get_manifest(self):
        """
        Get the manifest for this version.

        :return: Manifest for this version.
        """
        return self._api.manifest(self.project, self.revision)

    def get_builds(self):
        """
        Get all the builds that are a part of this version.

        :return: List of build that are a part of this version.
        """
        return self._api.builds_by_version(self.version_id)

    def is_patch(self):
        """
        Determine if this version from a patch build.

        :return: True if this version is a patch build.
        """
        return not self.version_id.startswith(self.project.replace('-', '_'))

    def is_completed(self):
        """
        Determine if this version has completed running tasks.

        :return: True if version has completed.
        """
        return self.status in COMPLETED_STATES

    def get_patch(self):
        """
        Get the patch information for this version.

        :return: Patch for this version.
        """
        if self.is_patch():
            return self._api.patch_by_id(self.version_id)
        return None

    def get_metrics(self, task_filter_fn=None):
        """
        Calculate the metrics for this version.

        Metrics are only available on versions that have finished running.

        :param task_filter_fn: function to filter tasks included for metrics, should accept a task
                               argument.
        :return: Metrics for this version.
        """
        if self.status != EVG_VERSION_STATUS_CREATED:
            return VersionMetrics(self).calculate(task_filter_fn)
        return None



# -*- encoding: utf-8 -*-
"""Commit Queue representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib


class CommitQueueItem(_BaseEvergreenObject):
    """Representation of an entry in a commit queue."""
    issue = evg_attrib('issue')
    modules = evg_attrib('modules')

    def __init__(self, json, api):
        super(CommitQueueItem, self).__init__(json, api)


class CommitQueue(_BaseEvergreenObject):
    """Representation of a commit queue from evergreen."""
    queue_id = evg_attrib('queue_id')

    def __init__(self, json, api):
        """
        Create an instance of a commit queue from evergreen json.

        :param json: Evergreen json representation of commit queue.
        :param api: Evergreen api object.
        """
        super(CommitQueue, self).__init__(json, api)

    @property
    def queue(self):
        """
        Retrieve the queue for this commit queue.
        :return: Queue of commits in the commit queue.
        """
        if not self.json['queue']:
            return []
        return [CommitQueueItem(item, self._api) for item in self.json['queue']]



# -*- encoding: utf-8 -*-
"""Get configuration about connecting to evergreen."""
from __future__ import absolute_import

from collections import namedtuple
import os

import yaml

EvgAuth = namedtuple('EvgAuth', ['username', 'api_key'])

DEFAULT_NETWORK_TIMEOUT_SEC = 5 * 60
DEFAULT_API_SERVER = 'https://evergreen.mongodb.com'
CONFIG_FILE_LOCATIONS = [
    os.path.expanduser(os.path.join('~', 'cli_bin', '.evergreen.yml')),
    os.path.expanduser(os.path.join('~', '.evergreen.yml')),
]


def read_evergreen_from_file(filename):
    """
    Read evergreen config from given filename.

    :param filename: Filename to read config.
    :return: Config read from file.
    """
    with open(filename, 'r') as fstream:
        return yaml.safe_load(fstream)


def read_evergreen_config():
    """
    Search known location for the evergreen config file.

    :return: First found evergreen configuration.
    """
    for filename in [filename for filename in CONFIG_FILE_LOCATIONS if os.path.isfile(filename)]:
        return read_evergreen_from_file(filename)

    return None


def get_auth_from_config(config):
    """
    Get the evergreen authentication from the specified config file.

    :param config: Evergreen configuration.
    :return: Authentication information for evergreen.
    """
    return EvgAuth(config['user'], config['api_key'])
